function [velhdr, res] = hdrmrl1(vels, wts, lambda, rho, mode, dims)
% HDRVEL = hdrmrl1( VELS, WTS, LAMBDA, RHO, MODE, DIMS )
%
% Inputs :
%  vels    - multi-venc velocity data with dimensions (x, y, z, t, dir, venc)
%  wts     - weight of each velocity measurement [0, 1]
%  rho     - augmented Lagrangian parameters
%  lambda  - l1-regularization parameter
%  mode    - l1-regularization function
%  dims    - dimensions to regularize across
%
% Example: If you want to use the HDR composition method to create a dataset
% with a TV penalty across spatial dimensions, then you would call:
%  velhdr = hdrmrl1(vels, wts, lambda, rho, 'tv', [1 2 3]);
%
% Notes :
% - Keeping track of the dimensions of the data is weird because of all the
%   reshaping going on in this function, so here are some reminders:
%    x - combined HDR dataset [x y z t vdir]
%    b - input multi-venc data [x y z t vdir venc]
%    Ax - dimensions [x y z t vdir nvencs]
%    Dx - dimensions [x y z t vdir ndims]
%
% Written by Christopher Sandino, 2017

% function prep
bdims = size(vels);   % input data dimensions
xdims = bdims(1:5);   % output data dimensions
ndims = length(dims); % number of dimensions that we are l1-constraining
nvencs = bdims(6);    % number of datasets that will be combined
dbmode = 1; % debug flag

% function definitions for image formation
Afun   = @(x) wts .* repmat(x, [1 1 1 1 1 nvencs]);
Atfun  = @(x) sum(wts.*x, 6);
AtAfun = @(x) Atfun(Afun(x));
if strcmp(mode, 'div') % l1-divergence penalty
    Dfun  = @(x) div(x, -1);
    Dtfun = @(x) div(x, 1);
elseif strcmp(mode, 'tv') % l1-total variation penalty
    Dfun  = @(x) grad(x, -1, dims);
    Dtfun = @(x) gradAdj(x, 1, dims);
elseif strcmp(mode, 'tv+div') % TV+divergence penalty
    Dfun = @(x) cat(6, grad(x,-1,dims), div(x, -1));
    Dtfun = @(x) x;
else
    error('hdrmrl1: specified mode not supported');
end
DtDfun = @(x) Dtfun(Dfun(x));
% pcg handle must take in and output a vector
Atilde = @(x) reshape( AtAfun(reshape(x,xdims)) + ...
    rho*DtDfun(reshape(x,xdims)), [], 1);

% ADMM parameters
lstol = 1e-5;
lsmaxiter = 100; % max l-s iterations
maxiter = 25; % max admm iterations

% pre-computed variables
b = wts .* vels;
Atb = Atfun(b);
z = zeros([xdims, ndims]);
u = zeros([xdims, ndims]);

if (dbmode) disp('hdrmrl1: begin ADMM'); end

for iter = 1:maxiter
    % x-update
    btilde = Atb + rho*Dtfun(z-u);
    x = reshape( pcg(Atilde, btilde(:), lstol, lsmaxiter), xdims );
    % z-update
    Dx = Dfun(x);
    z = shrinkage(Dx + u, lambda/rho);
    % u-update
    u = u + Dx - z;
    % compute residual
    res(iter) = 0.5*norm(reshape(Afun(x)-b,[],1))^2 + lambda*norm(Dx(:), 1);

    % debug print statements
    if dbmode
        disp([' iteration ' num2str(iter)]);
        disp([' residual = ' num2str(res(iter))]);
    end
end

velhdr = x;

end
