%% hdrflow_workspace.m
% Christopher Sandino
% 9 February 2017

%% script prep

clear; clc;

addpath('./phantom'); % phantom libraries
addpath('./fun'); % function libraries


%% generate phantom data

% phantom parameters
dims = [512 512];
resize_dims = [64 64]; % must be smaller than 0.5*dims
nslices = 64;
nframes = 30;
venc = [1.75 0.75 0.5]; % [m/s]
peak_vels = [-1 0.1]; % [m/s]
snr = 5; % signal-to-noise ratio

% generate reference phantom data
[rvel, rmag, rmasks] = mvflow_sim(dims, nframes, peak_vels);

% simulate MRI data from reference data for multiple vencs
for v = 1:length(venc)
    [vel, im, masks] = gen_4dflow_phantom(rvel, rmag, rmasks, ...
        resize_dims, nslices, venc(v), snr);
    vels(:,:,:,:,1,v) = angle(im)*venc(v)/pi;
    mags(:,:,:,:,1,v) = abs(im);
    clear im;
end


%% combine multi-venc acqs into one HDR dataset

% compute wrapping weights
wrapwts = ones(size(vels));
for v = 1:length(venc)
    wrapwts(:,:,:,:,:,v) = 1-logistic(abs(vels(:,:,:,:,:,1)), venc(v)-0.15, 30);
end

% compute magnitude-derived weights
magwts = mags / max(mags(:));
magwts = magwts ./ permute(repmat(venc', [1 size(vels(:,:,:,:,:,1))]), [2 3 4 5 6 1]);
magwts = magwts / max(magwts(:));

% compute combined weights
wts = wrapwts .* magwts;

% HDR combine
velhdr1 = sum(wts.*vels, 6) ./ sum(wts, 6); % L-S
velhdr2 = hdrmrl1(vels, wts, 0.05, 1, 'tv', [1 2 3 4]); % L-S + TV

% plot images
veltruth = permute(repmat(vel, [1 1 1 nslices]),[1 2 4 3]);
as(cat(2,veltruth, velhdr1,velhdr2));


% plot vels, mags, weight maps
figure; hold on;
slice = 32;
cph = 9;
magmax = max(reshape(mags(:,:,:,:,1),[],1));
for v = 1:length(venc)
    % velocity map
    subplot(3, 5, 5*(v-1)+1); imagesc(vels(:,:,slice,cph,1,v), [-0.5 0.5]);
        axis equal tight;
    % magnitude
    subplot(3, 5, 5*(v-1)+2); imagesc(mags(:,:,slice,cph,1,v), [0 magmax]);
        axis equal tight; colormap gray;
    % wrapping weights
    subplot(3, 5, 5*(v-1)+3); imagesc(wrapwts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
    % magnitude weights
    subplot(3, 5, 5*(v-1)+4); imagesc(magwts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
    % combined weights
    subplot(3, 5, 5*(v-1)+5); imagesc(wts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
end

%% compute biomarkers for reference and simulated data

% allocate memory
nves = size(masks, 3);
area = zeros(nves, 1);
pv = zeros(nves, 1);
mv = zeros(nves, 1);
totflow = zeros(nves, 1);

% pre-compute
sres = 0.1; % [cm] assume 1 mm isotropic resolution
pixarea = sres^2; % area per pixel [cm^2]
tres = 1 / nframes; % [s] assume 60 BPM (1 s/cycle)

for ves = 1:nves % loop over vessels
    maskedvel = 100*repmat(masks(:,:,ves),[1 1 nframes]) .* vel; % [cm/s]

    % peak velocity (cm/s)
    pv(ves) = max(abs(maskedvel(:)));
    
    % flow rate (mL/s)
    npix = sum(sum(masks(:,:,ves))); % number of pixels in ROI
    area(ves) = npix*pixarea; % cross-sectional area of vessel
    Q(ves) = area(ves);
    
    % total flow over cardiac cycle (mL)
    % totflow(ves) = sum(sum(sum(Q)));
end

