function [VEL, MAG, MASKS] = flow_sim(dims, nframes)
%% FLOW_SIM Simulates arterial flow through cylindrical vessel
%
% Creates a digital flow phantom using the Womersley method - solves
% simplifed Navier-Stokes equations for arterial flow field in 2D. Because 
% it is ideal flow the velocity vector is entirely into the plane.
%
% Inputs: 
%   dims: 2D array giving size of the phantom, this might not work
%       if the sizes aren't equal (not tested)
%   num_frames: number of timeframes (cardiac phases), defaults to 30
%   
% Outputs:
%   VEL: matrix with the velocities
%   MAG: matrix with magnitude values
%   MASKS: mask of each vessel (dims x 4)
%
% Code by Michael Loecher (https://github.com/mloecher/4dflow-lapunwrap)
%
% Modified by Christopher M. Sandino, March 2016
%

disp('flow_sim: generating 2D Womersley velocity profile...');

%% Womersley arterial flow coefficients

C0 = -25;
D = 400;
C = [7.58 5.41 1.52 0.52 0.83 0.69 0.26 0.54 0.27 0.10];
Phi = [-174 89 -22 -34 -127 135 152 44 -72 11]*pi/180;

coeff = D.*C.*exp(1i.*Phi);

mu = 4e-3; % viscosity Pa*s
ro = 1.06e3; % density kg/m^3
nu = mu/ro; % viscosity m^2/s
omega = 2*pi;


%% Location of the vessels (Assumed FOV = 30cm)

ves_sep_m = .025; % Seperation (meters)
ves_sep  = ves_sep_m*dims(1)/.3; % convert to pixels

n_ves = 4; % number of vessels in image
ves_rad = [.02 .015 .01 .005]; % meters

x_cent = dims(1)/2;
ves_x = [x_cent+ves_sep x_cent+ves_sep x_cent-ves_sep x_cent-ves_sep];
y_cent = dims(2)/2;
ves_y = [y_cent+ves_sep y_cent-ves_sep y_cent+ves_sep y_cent-ves_sep];

ves_rad_p = ves_rad .* dims(1)/.3; % FOV = 30 cm, assuming isotropic voxels


%% Add background signal

MAG = zeros(dims, 'single');
meshspace1 = linspace(-.5, .5, dims(1));
meshspace2 = linspace(-.5, .5, dims(2));
[XX, YY] = meshgrid(meshspace1, meshspace2);
R = sqrt(XX.^2 + YY.^2);

MAG(R<.45) = 0.7;


%% Calculate velocity values for each vessel using Womersley method

VEL = zeros([dims nframes], 'single');
MASKS = zeros([dims 4], 'single');

for vi = 1:n_ves
    for x=floor(ves_x(vi)-ves_rad_p(vi)-2):floor(ves_x(vi)+ves_rad_p(vi)+2)
        for y=floor(ves_y(vi)-ves_rad_p(vi)-2):floor(ves_y(vi)+ves_rad_p(vi)+2)
            
            rad1= sqrt( (x-ves_x(vi))^2+(y-ves_y(vi))^2);
            if(rad1 < ves_rad_p(vi))
                MAG(x,y) = 1.0;
                MASKS(x,y,vi) = 1.0;
                for p = 1:nframes
                    
                    t_sel = (p - 1)/nframes;
                    alpha = sqrt(ro*omega*ves_rad(vi)^2/mu);
                    rhat = rad1/ves_rad_p(vi);
                    
                    n = 1:10;
                    ex = 1i^(3/2).*sqrt(n).*alpha;
                    vt = coeff.*(ves_rad(vi)^2./1j./n./mu./alpha^2).*(1 - besselj(0,ex.*rhat)./besselj(0,ex) ).*exp(1j.*n.*omega.*t_sel);
                    Velocity = sum(vt);
                    Velocity = Velocity + C0*ves_rad(vi)^2/4/mu*(1-rhat^2);
                    VEL(x,y,p) = -real(Velocity);
                    
                end
            end
            
        end
    end
end


%% Add some minor blurring
% Even with the Fourier resizing we do later and the higher resolution
% phantom, the jagged and infinetely sharp edges aren't realistic, so we do
% very minor blurring to fix the vessel walls.

G = fspecial('gaussian', [9 9], 1);

MAG = imfilter(MAG, G, 'same');

for i = 1:nframes
    VEL(:,:,i) = imfilter(VEL(:,:,i), G, 'same');
end


%% Normalize peak velocities 
%  Note that this changes pressure->velocity relationship a little from
%  Womersley, but this should be unrelated to unwrapping performance.
%  This also means that smaller vessels have different flow profiles than
%  the larger, but this is realistic.

for i = 1:n_ves
    xr=floor(ves_x(i)-ves_rad_p(i)-2):floor(ves_x(i)+ves_rad_p(i)+2);
    yr=floor(ves_y(i)-ves_rad_p(i)-2):floor(ves_y(i)+ves_rad_p(i)+2);
    data = VEL(xr, yr, :);
    VEL(xr, yr, :) = VEL(xr, yr, :)./max(data(:));
end

disp('flow_sim: done!');

end

