function [VEL, IM, MASKS, NOISE] = gen_4dflow_phantom(vel, mag, masks, dims, nslices, venc, snr)
%% GEN_4DFLOW_PHANTOM Generates a 4D flow MRI phantom for unwrapping
% 
% Velocity data is simulated as volumetric MR data by adding complex 
% noise and cropping in k-space, and simulating an excitation profile in 
% the z-direction.
%
% Args:
%     dims: 2D array giving size of the phantom, this might not work
%       if the sizes aren't equal (not tested)
% 
% Returns:
%     VEL: matrix with the velocities
%     MAG: matrix with magnitude values
%     MASKS: mask of each vessel (dims x 4) 
%     NOISE:

% Code by Michael Loecher (4dflow_lapunwrap toolbox)
%
% Modified by Christopher Sandino, March 2016


%% Make the data complex

nframes = size(vel, 3);
VELr = vel*pi/venc; % convert to angle in radians
IM = repmat(mag, [1 1 nframes]) .* exp(1j*VELr);


%% Resize image
%  Assumes that the 2D input is high resolution for more accuracy
%  Crop in k-space for realism (some ringing at vessel walls and partial 
%  volume) Going back to image space isnt strictly necessary, but makes 
%  excitation simulation slightly more realistic.

lpf_fac = 2; % low pass filt factor - make larger for more ringing/partial vol
orig_dims = [size(VELr, 1) size(VELr, 2)];
lpf_dims = dims * 2;

% resize complex phase-contrast data
K = ifftshift(fft2(fftshift(IM)));
r1 = (1:lpf_dims(1)) + (size(K,1)-lpf_dims(1))/2;
r2 = (1:lpf_dims(2)) + (size(K,2)-lpf_dims(2))/2;
mod1 = (size(K,1)-lpf_dims(1)) / size(K,1);
mod2 = (size(K,2)-lpf_dims(2)) / size(K,2);
K = K(r1, r2, :) * (mod1.^(mod1~=0)) * (mod2.^(mod2~=0));
IM = ifftshift(ifft2(fftshift(K)));

% Also resize the reference velocity so we have a 'truth'
K = ifftshift(fft2(fftshift(vel)));
mod1 = lpf_dims(1) / size(K,1);
mod2 = lpf_dims(2) / size(K,2);
K = K(r1, r2, :) * (mod1.^(mod1~=0)) * (mod2.^(mod2~=0));
VELr = real(ifftshift(ifft2(fftshift(K))));


%% Crop
% This is to speed up the algorithm, crop the data to only include the 
% vessels. 

cr1 = (1:dims(1)) + (size(K,1)-dims(1))/2;
cr2 = (1:dims(2)) + (size(K,2)-dims(2))/2;
IM = IM(cr1, cr2, :);
VEL = VELr(cr1, cr2, :);


%% Resize the masks with imresize
%  because we dont want ringing or anything in these

MASKS = zeros([lpf_dims 4], 'single');
for i = 1:size(masks, 3)
   MASKS(:,:,i) = imresize(masks(:,:,i), lpf_dims); 
end

MASKS(MASKS<0.5) = 0;
MASKS(MASKS>0) = 1;
MASKS = MASKS(cr1, cr2, :);
MASKS = logical(MASKS);


%% Make 3D

IM = permute(IM, [1 2 4 3]);
IM = repmat(IM, [1 1 nslices 1]);

%% Excitation 
%  This is important to make the unwrapping simulation realistic because
%  of the importance of boundary conditions.  Because we are using FFTs the
%  periodicity in the z-direction is unrealistic compared to an in vivo
%  case, so we add a fake and idealized excitation profile where the signal
%  goes to zero.  Subsequently, we dont include these areas in the
%  analysis.

cutoff = 6;

excite = ones([nslices 1], 'single');
excite(1:cutoff) = 0;
excite(end-cutoff-1:end) = 0;
w = gausswin(9);
w = w./sum(w(:));
excite = conv(excite,w,'same');

IM = IM .* repmat(permute(excite, [2 3 1]), [size(IM,1) size(IM,2) 1 size(IM,4)]);


%% Add complex Gaussian noise 

sigma = sqrt(numel(IM))/snr;
NOISE = sigma*( randn(size(IM)) + 1j*randn(size(IM)) ); 
K = ifftshift(fftn(fftshift(IM)));
K = K + NOISE;
IM = ifftshift(ifftn(fftshift(K)));


end



