function [VEL, MAG, MASKS] = mvflow_sim(dims, nframes, peak_vels)
%% MVFLOW_SIM Simulates arterial/venous flow through cylindrical vessel
%
% Creates a digital flow phantom using the Womersley method - solves
% simplifed Navier-Stokes equations for arterial flow field in 2D. Because 
% it is ideal flow the velocity vector is entirely into the plane. Modified
% from original implementation to include both arteries and "vessels"
% (vessels have a scaled arterial velocity profile - not realistic, but
% wanted it for testing multi-venc schemes)
%
% Inputs: 
%   dims: 2D array giving size of the phantom, this might not work
%       if the sizes aren't equal (not tested)
%   num_frames: number of timeframes (cardiac phases), defaults to 30
%   peak_vels: peak velocities for [arteries, veins]
%   
% Outputs:
%   VEL: matrix with the velocities
%   MAG: matrix with magnitude values
%   MASKS: mask of each vessel (dims x 8)
%
% Code by Michael Loecher (https://github.com/mloecher/4dflow-lapunwrap)
%
% Modified by Christopher M. Sandino, March 2016
%

disp('mvflow_sim: generating 2D Womersley velocity profile...');

%% Womersley arterial flow coefficients

C0 = -25;
D = 400;
C = [7.58 5.41 1.52 0.52 0.83 0.69 0.26 0.54 0.27 0.10];
Phi = [-174 89 -22 -34 -127 135 152 44 -72 11]*pi/180;

coeff = D.*C.*exp(1i.*Phi);

mu = 4e-3; % viscosity Pa*s
ro = 1.06e3; % density kg/m^3
nu = mu/ro; % viscosity m^2/s
omega = 2*pi;


%% Location of the vessels (Assumed FOV = 30cm)

% define an array of vessels
n_row = 2;
n_col = 4;
n_ves = n_row * n_col;

FOV = 0.3; % field of view (meters)
ves_sep = .005; % Seperation (meters)
ves_sep_p  = ves_sep*dims(1)/FOV; % convert to pixels
ves_rad = repmat([.02 .015 .01 .005], 1, n_row); % meters
ves_rad_p = ves_rad * dims(1)/FOV;
L = sum(ves_rad_p(1:n_col)) + (n_ves-1)*ves_sep_p; % length spanned by each row

% compute x-coordinate of each center of each vessel
x_cent = dims(1)/2;
ves_x(1:n_col) = x_cent - ves_rad_p(1) - ves_sep_p;
ves_x(n_col+1:n_ves) = x_cent + ves_rad_p(1) + ves_sep_p;

% compute y-coordinate of each center of each vessel
y_cent = dims(2)/2;
ves_y(1) = y_cent-L/2;
ves_y(1+n_col) = y_cent-L/2;
for i = 2:n_col
    ves_y(i) = ves_y(i-1) + ves_rad_p(i-1) + ves_sep_p + ves_rad_p(i);
    ves_y(i+n_col) = ves_y(i);
end


%% Add background signal

MAG = zeros(dims, 'single');
meshspace1 = linspace(-.5, .5, dims(1));
meshspace2 = linspace(-.5, .5, dims(2));
[XX, YY] = meshgrid(meshspace1, meshspace2);
R = sqrt(XX.^2 + YY.^2);

MAG(R<.45) = 0.7;


%% Calculate velocity values for each vessel using Womersley method

VEL = zeros([dims nframes], 'single');
MASKS = zeros([dims 4], 'single');

for vi = 1:n_ves
    for x=floor(ves_x(vi)-ves_rad_p(vi)-2):floor(ves_x(vi)+ves_rad_p(vi)+2)
        for y=floor(ves_y(vi)-ves_rad_p(vi)-2):floor(ves_y(vi)+ves_rad_p(vi)+2)
            
            rad1= sqrt( (x-ves_x(vi))^2+(y-ves_y(vi))^2);
            if(rad1 < ves_rad_p(vi))
                MAG(x,y) = 1.0;
                MASKS(x,y,vi) = 1.0;
                for p = 1:nframes
                    
                    t_sel = (p - 1)/nframes;
                    alpha = sqrt(ro*omega*ves_rad(vi)^2/mu);
                    rhat = rad1/ves_rad_p(vi);
                    
                    n = 1:10;
                    ex = 1i^(3/2).*sqrt(n).*alpha;
                    vt = coeff.*(ves_rad(vi)^2./1j./n./mu./alpha^2).*(1 - besselj(0,ex.*rhat)./besselj(0,ex) ).*exp(1j.*n.*omega.*t_sel);
                    Velocity = sum(vt);
                    Velocity = Velocity + C0*ves_rad(vi)^2/4/mu*(1-rhat^2);
                    VEL(x,y,p) = -real(Velocity);
                    
                end
            end
            
        end
    end
end


%% Add some minor blurring
% Even with the Fourier resizing we do later and the higher resolution
% phantom, the jagged and infinetely sharp edges aren't realistic, so we do
% very minor blurring to fix the vessel walls.

G = fspecial('gaussian', [9 9], 1);

MAG = imfilter(MAG, G, 'same');

for i = 1:nframes
    VEL(:,:,i) = imfilter(VEL(:,:,i), G, 'same');
end


%% Normalize peak velocities 
%  Note that this changes pressure->velocity relationship a little from
%  Womersley, but this should be unrelated to unwrapping performance.
%  This also means that smaller vessels have different flow profiles than
%  the larger, but this is realistic.

% weights of peak velocities [arteries veins]
wts = [repmat(peak_vels(1), 1, n_col) repmat(peak_vels(2), 1, n_col)];

for i = 1:n_ves
    xr=floor(ves_x(i)-ves_rad_p(i)-2):floor(ves_x(i)+ves_rad_p(i)+2);
    yr=floor(ves_y(i)-ves_rad_p(i)-2):floor(ves_y(i)+ves_rad_p(i)+2);
    data = VEL(xr, yr, :);
    VEL(xr, yr, :) = wts(i)*VEL(xr, yr, :)./max(data(:));
end

disp('mvflow_sim: done!');

end

