
%% Load data

clear; clc; close all;

dbplot = 0;
top = '/home/sandino/data/Studies_4dflowRenal/2016_04_01_Renal_MultiVENC/';

% set new dictionary so that necessary 4D flow dicom fields get written
dictionary = [top 'new-dicom-dict.txt']; % dicom dictionary
dicomdict('set', dictionary);

% load dicom header information
hdr = dicominfo( [top 'Ser6_V50/dcm/Cph_000_Sec_001.si'] );

% data dimensions
nx = hdr.Height; % 320
ny = hdr.Width;  % 224
nslices = 160;   % 1 - 160 (hdr.ImagesInAcquisition * ZIPfac)
ncph = hdr.CardiacNumberOfImages; % 0 - 11
vencs = [120, 50];
nvencs = length(vencs);
series = [7, 6];

% allocate memory
vels = zeros(nx, ny, nslices, ncph, 3, length(vencs), 'single');
mags = zeros(nx, ny, nslices, ncph, 1, length(vencs), 'single');

% read in data
disp('test_invivo.m: loading dicom data'); tic;
for venc = 1:nvencs
    for cph = 1:ncph
        for slice = 1:nslices
            folder = [top 'Ser' num2str(series(venc)) '_V' num2str(vencs(venc)) '/dcm/'];
            filename = [folder 'Cph_' num2str(cph-1, '%03d') '_Sec_' num2str(slice, '%03d')];
            mags(:,:,slice,cph,1,venc) = dicomread( [filename '.mag'] );
            vels(:,:,slice,cph,1,venc) = dicomread( [filename '.rl'] );
            vels(:,:,slice,cph,2,venc) = dicomread( [filename '.ap'] );
            vels(:,:,slice,cph,3,venc) = dicomread( [filename '.si'] );
        end
    end
end
toc;


%% phase-contrast corrections

disp('test_invivo.m: doing phase-contrast corrections');

disp('  scaling...'); tic;
vels = vels / 512 * pi; % map to [-pi, +pi]
toc;

disp('  phase unwrapping...'); tic;
for dir = 1:3
    vels(:,:,:,:,dir,1) = lapunwrap(vels(:,:,:,:,dir,1));
end
toc;

disp('  eddy current correction...'); tic;
for venc = 1:length(vencs)
    emap(:,:,:,1,:,1) = pcerror(mags(:,:,:,:,:,venc), vels(:,:,:,:,:,venc));
    vels(:,:,:,:,:,venc) = bsxfun(@minus, vels(:,:,:,:,:,venc), emap);
end
toc;

disp('  re-scaling...');
for venc = 1:length(vencs)
    % convert phase data to velocity [-venc, +venc]
    vels(:,:,:,:,:,venc) = vels(:,:,:,:,:,venc)*vencs(venc)/pi;
end


%% compute weights for HDR algorithm

disp('test_invivo.m: computing HDR weights');

% compute wrapping weights
thresh = 0.3; % reject velocities within [-venc_i, + venc_i]
wrapwts = ones(size(vels));
for v = 2:nvencs
    wrapwts(:,:,:,:,:,v) = 1-logistic(abs(vels(:,:,:,:,:,1)), (1-thresh)*vencs(v), 30);
end

% compute magnitude-derived weights
magwts = mags / max(mags(:));
magwts = magwts ./ permute(repmat(vencs', [1 size(vels(:,:,:,:,:,1))]), [2 3 4 5 6 1]);
magwts = magwts / max(magwts(:));

% compute combined weights
wts = wrapwts .* magwts;

% plot vels, mags, weight maps
if dbplot
    figure; hold on;
    slice = 74;
    cph = 6;
    magmax = max(reshape(mags(:,:,slice,cph,:,1),[],1));
    for v = 1:nvencs
        % velocity map
        subplot(nvencs, 5, 5*(v-1)+1); imagesc(vels(:,:,slice,cph,3,v), [-50 50]);
            axis equal tight;
        % magnitude
        subplot(nvencs, 5, 5*(v-1)+3); imagesc(mags(:,:,slice,cph,1,v), [0 magmax]);
            axis equal tight; colormap gray;
        % wrapping weights
        subplot(nvencs, 5, 5*(v-1)+2); imagesc(wrapwts(:,:,slice,cph,3,v), [0 0.5]);
            axis equal tight; colormap gray;
        % magnitude weights
        subplot(nvencs, 5, 5*(v-1)+4); imagesc(magwts(:,:,slice,cph,3,v), [0 0.5]);
            axis equal tight; colormap gray;
        % combined weights
        subplot(nvencs, 5, 5*(v-1)+5); imagesc(wts(:,:,slice,cph,3,v), [0 0.5]);
            axis equal tight; colormap gray;
    end
end


%% combine multi-venc datasets into one composite HDR dataset

velhdr = sum(wts.*vels, 6) ./ sum(wts, 6); % L-S
% velhdr2 = hdrmrl1(vels, wts, 0.05, 1, 'tv', [1 2 3 4]); % L-S + TV
% velhdr3 = hdrmrl1(vels, wts, 0.05, 1, 'div', []); % L-S + Div
% velhdr4 = hdrmrl1(vels, wts, 0.05, 1, 'tv+div', [4]); % L-S + TV(time) + Div


%% write out data

venc = 1;
ext = {'.rl', '.ap', '.si'};

disp('test_invivo.m: writing out dicom data'); tic;

for cph = 1:ncph
    for slice = 1:nslices
        folder = [top 'Ser' num2str(series(venc)) ...
            '_V' num2str(vencs(venc)) '/dcm/'];
        filename = [folder 'Cph_' num2str(cph-1, '%03d') ...
            '_Sec_' num2str(slice, '%03d')];
        newfolder = [top 'Ser' num2str(series(venc)) ...
            '_V' num2str(vencs(venc)) '/hdrdcm/'];
        newfilename = [newfolder 'Cph_' num2str(cph-1, '%03d') ...
            '_Sec_' num2str(slice, '%03d')];
        
        disp(['  Writing out ' newfilename]);
        
        % copy complex difference data
        copyfile([filename '.cd'], [newfilename '.cd']);
        
        % write out magnitude data
        hdr = dicominfo( [filename '.mag'] );
        hdr.PatientName.FamilyName = 'HDR'; % patient name
        hdr.PatientName.BirthName = 'TEST1';
        hdr.PatientID = 'CMS'; % patient ID
        % write out data
        im = int16( mags(:,:,slice,cph,1,venc) );
        dicomwrite( im, [newfilename, '.mag'], ...
            hdr, 'CreateMode', 'copy', 'WritePrivate', true ); % header stuff
        
        % write out velocity data
        for dir = 1:3
            % read dicom header
            hdr = dicominfo( [filename, ext{dir}] );
            % change patient name
            hdr.PatientName.FamilyName = 'HDR'; % patient name
            hdr.PatientID = 'CMS'; % patient ID
            % write out data
            im = velhdr(:,:,slice,cph,dir,venc)*512/vencs(venc); % re-scale
            im = int16( im ); % convert to int16
            dicomwrite( im, [newfilename, ext{dir}], ...
                hdr, 'CreateMode', 'Copy' ); % header stuff
        end
    end
end
toc;