%% hdrflow_workspace.m
% Christopher Sandino
% 9 February 2017

%% script prep

clear; clc;

addpath('./phantom'); % phantom libraries
addpath('./fun'); % function libraries


%% generate phantom data

% MODIFY THESE !

% phantom parameters
dims = [512 512];
resize_dims = [64 64]; % must be smaller than 0.5*dims
nslices = 64;
nframes = 30;
venc = [1.5 0.75 0.5]; % [m/s]
peak_vels = [-1 0.05]; % [m/s]
snr = 5; % signal-to-noise ratio

% generate reference phantom data
[rvel, rmag, rmasks] = mvflow_sim(dims, nframes, peak_vels);

% simulate MRI data from reference data for multiple vencs
for v = 1:length(venc)
    [vel, im, masks] = gen_4dflow_phantom(rvel, rmag, rmasks, ...
        resize_dims, nslices, venc(v), snr);
    vels(:,:,:,:,1,v) = angle(im)*venc(v)/pi;
    mags(:,:,:,:,1,v) = abs(im);
    clear im;
end

% plot images for each venc
cph = 9; % temporal phase
slice = 32; % slice
figure; imagesc(cat(2, vels(:,:,slice,cph,1,1), ...
    vels(:,:,slice,cph,1,2), ...
    vels(:,:,slice,cph,1,3)));
title(['Multi-venc velocity data']);


%% combine multi-venc acqs into one HDR dataset

% compute wrapping weights
thresh = 0.3;
wrapwts = ones(size(vels));
for v = 2:length(venc)
    wrapwts(:,:,:,:,:,v) = 1-logistic(abs(vels(:,:,:,:,:,1)), (1-thresh)*venc(v), 25);
end

% compute magnitude-derived weights
magwts = mags / max(mags(:));
magwts = magwts ./ permute(repmat(venc', [1 size(vels(:,:,:,:,:,1))]), [2 3 4 5 6 1]);
magwts = magwts / max(magwts(:));

% compute combined weights
wts = wrapwts .* magwts;

% HDR combine
disp('HDR fusion (lambda = 0)'); tic;
velhdr1 = sum(wts.*vels, 6) ./ sum(wts, 6); % L-S
toc;

disp('HDR fusion + TV regularization'); tic;
velhdr2 = hdrmrl1(vels, wts, 0.01, 1, 'tv', [1 2 3 4]); % L-S + TV
toc;

% plot images
cph = 9; % temporal phase
slice = 32; % slice
figure; imagesc(cat(2, vel(:,:,cph), ... % ground truth
    vels(:,:,slice,cph,1,1), ... % high venc data
    velhdr1(:,:,slice,cph), ... % HDR images
    velhdr2(:,:,slice,cph))); % HDR + TV images
title('Images (Ground truth, High Venc, HDR, HDR+TV)');

% plot vels, mags, weight maps
figure; hold on;
slice = 32;
cph = 9;
magmax = max(reshape(mags(:,:,slice,cph,:,1),[],1));
for v = 1:length(venc)
    % velocity map
    subplot(3, 5, 5*(v-1)+1); imshow(vels(:,:,slice,cph,1,v), [-0.2 0.2]);
        axis equal tight;
    % magnitude
    subplot(3, 5, 5*(v-1)+3); imshow(mags(:,:,slice,cph,1,v), [5 magmax]);
        axis equal tight; colormap gray;
    % wrapping weights
    subplot(3, 5, 5*(v-1)+2); imshow(wrapwts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
    % magnitude weights
    subplot(3, 5, 5*(v-1)+4); imshow(magwts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
    % combined weights
    subplot(3, 5, 5*(v-1)+5); imshow(wts(:,:,slice,cph,1,v), [0 1]);
        axis equal tight; colormap gray;
end
title('Weight maps');


